@@ -22,11 +22,11 @@ Params::Validate::Strict - Validates a set of parameters against a schema
 
 =head1 VERSION
 
-Version 0.26
+Version 0.27
 
 =cut
 
-our $VERSION = '0.26';
+our $VERSION = '0.27';
 
 =head1 SYNOPSIS
 
@@ -139,7 +139,7 @@ Custom types allow you to define validation rules once and reuse them throughout
 making your validation logic more maintainable and readable.
 
 Each custom type is defined as a hash reference containing the same validation rules available for regular parameters
-(C<type>, C<min>, C<max>, C<matches>, C<memberof>, C<notmemberof>, C<callback>, etc.).
+(C<type>, C<min>, C<max>, C<matches>, C<memberof>, C<enum>, C<notmemberof>, C<callback>, etc.).
 
   my $custom_types = {
     email => {
@@ -260,6 +260,10 @@ Note that C<memberof> cannot be combined with C<min> or C<max> constraints as th
 serve conflicting purposes - C<memberof> defines an explicit whitelist while C<min>/C<max>
 define ranges.
 
+=item * C<enum>
+
+Same as C<memberof>.
+
 =item * C<notmemberof>
 
 The parameter must not be a member of the given arrayref (blacklist).
@@ -707,6 +711,15 @@ Cross-validations can access nested structures and optional fields:
     }
   };
 
+=item * metadata
+
+Fields starting with <_> are generated by L<App::Test::Generator::SchemaExtractor>,
+and are currently ignored.
+
+=item * schematic
+
+TODO: gives an idea of what the field will be, e.g. C<filename>.
+
 All cross-validations must pass for the overall validation to succeed.
 
 =back
@@ -1254,7 +1267,7 @@ sub validate_strict
 						}
 						$invalid_args{$key} = 1;
 					}
-				} elsif($rule_name eq 'memberof') {
+				} elsif(($rule_name eq 'memberof') || ($rule_name eq 'enum')) {
 					if(!defined($value)) {
 						next;	# Skip if string is undefined
 					}
@@ -1414,6 +1427,16 @@ sub validate_strict
 					# Handled inline
 				} elsif($rule_name eq 'description') {
 					# A la, Data::Processor
+				} elsif($rule_name =~ /^_/) {
+					# Ignore internal/metadata fields from schema extraction
+				} elsif($rule_name eq 'semantic') {
+					if($rule_value eq 'unix_timestamp') {
+						if($value < 0 || $value > 2147483647) {
+							error($logger, 'Invalid Unix timestamp: $value');
+						}
+					} else {
+						_warn($logger, "semantic type $rule_value is not yet supported");
+					}
 				} elsif($rule_name eq 'schema') {
 					# Nested schema Run the given schema against each element of the array
 					if($rules->{'type'} eq 'arrayref') {
@@ -1564,6 +1587,9 @@ sub _number_of_characters
 	$value = decode_utf8($value) unless utf8::is_utf8($value);
 
 	# Count grapheme clusters (visible characters)
+	# The pseudo-operator () = forces list context to count matches
+	# return scalar( () = $value =~ /\X/g );
+
 	return Unicode::GCString->new($value)->length();
 }
 
